local ADDON_NAME, PRIVATE_TABLE = ...

local _G = getfenv(0)
local UIParent = _G.UIParent
local WorldFrame = _G.WorldFrame

local GetTime = _G.GetTime
local IsMounted = _G.IsMounted
local CreateFrame = _G.CreateFrame
local MouselookStop = _G.MouselookStop
local IsMouselooking = _G.IsMouselooking
local InCombatLockdown = _G.InCombatLockdown
local ClearOverrideBindings = _G.ClearOverrideBindings
local SetOverrideBindingClick = _G.SetOverrideBindingClick

local LibStub = _G.LibStub
local AceAddon = LibStub("AceAddon-3.0")

PRIVATE_TABLE.addon = PRIVATE_TABLE.addon or {}

local ACTION_DOUBLE_WAIT = 0.4
local MIN_ACTION_DOUBLECLICK = 0.05
local LOOT_A_RANG_1_ID = 60854
local LOOT_A_RANG_2_ID = 109167

local Addon = AceAddon:NewAddon(PRIVATE_TABLE.addon, ADDON_NAME, "AceEvent-3.0", "AceHook-3.0")

function Addon:OnInitialize()
	if not PRIVATE_TABLE.secureActionButton then
		local button = CreateFrame("Button", "LootARangMatic", UIParent, "SecureActionButtonTemplate")
		button:SetPoint("LEFT", UIParent, "RIGHT", 10000, 0)
		button:Hide()
		button:SetFrameStrata("LOW")
		button:EnableMouse(true)
		button:RegisterForClicks("RightButtonUp")
		button:SetAttribute("action", nil)
		
		button:SetScript("PostClick", function(self, mouse_button, is_down)
			if PRIVATE_TABLE.bindingOn and not PRIVATE_TABLE.inCombat and not InCombatLockdown() then
				ClearOverrideBindings(self)
				PRIVATE_TABLE.bindingOn = false
			else
				PRIVATE_TABLE.bindingStuck = true
			end
		end)
		
		PRIVATE_TABLE.secureActionButton = button
	end
end

function Addon:SetupSecureAction()
	local name = ""
	if GetItemCount(LOOT_A_RANG_1_ID, false, false) >= 1 then
		name = GetItemInfo(LOOT_A_RANG_1_ID)
		PRIVATE_TABLE.secureActionButton:SetAttribute("type", "item")
		PRIVATE_TABLE.secureActionButton:SetAttribute("item", name)
	elseif GetItemCount(LOOT_A_RANG_2_ID, false, false) >= 1 then
		name = GetItemInfo(LOOT_A_RANG_2_ID)
		PRIVATE_TABLE.secureActionButton:SetAttribute("type", "item")
		PRIVATE_TABLE.secureActionButton:SetAttribute("item", name)
	elseif IsSpellKnown(125050) then
		name = GetSpellInfo(125050)
		PRIVATE_TABLE.secureActionButton:SetAttribute("type", "spell")
		PRIVATE_TABLE.secureActionButton:SetAttribute("spell", name)
	else
		PRIVATE_TABLE.secureActionButton:SetAttribute("type", "item")
		PRIVATE_TABLE.secureActionButton:SetAttribute("item", name)
	end
end

function Addon:OnEnable()
	self:SetupSecureActionWhenSafe()
	self:RegisterEvent("BAG_UPDATE", "SetupSecureActionWhenSafe")
	self:RegisterEvent("SPELLS_CHANGED", "SetupSecureActionWhenSafe")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnRegenDisabled")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnRegenEnabled")
	self:RegisterEvent("PLAYER_STARTED_MOVING", "OnPlayerMoveStart")
	self:RegisterEvent("PLAYER_STOPPED_MOVING", "OnPlayerMoveStop")
	self:SecureHookScript(WorldFrame, "OnMouseDown", "OnMouseDown")
end

function Addon:OnDisable()
	self:UnregisterEvent("BAG_UPDATE")
	self:UnregisterEvent("SPELLS_CHANGED")
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("PLAYER_STARTED_MOVING")
	self:UnregisterEvent("PLAYER_STOPPED_MOVING")
	self:Unhook(WorldFrame, "OnMouseDown")
end

function Addon:SetupSecureActionWhenSafe()
	if not PRIVATE_TABLE.inCombat and not InCombatLockdown() then
		self:SetupSecureAction()
	else
		PRIVATE_TABLE.bindingNeedUpdate = true;
	end
end

function Addon:OnRegenDisabled()
	PRIVATE_TABLE.inCombat = true
end

function Addon:OnRegenEnabled()
	PRIVATE_TABLE.inCombat = false
	if PRIVATE_TABLE.bindingStuck then
		ClearOverrideBindings(PRIVATE_TABLE.secureActionButton)
		PRIVATE_TABLE.bindingStuck = false
		PRIVATE_TABLE.bindingStuck = false
	end
	if PRIVATE_TABLE.bindingNeedUpdate then
		self:SetupSecureAction()
	end
end

function Addon:OnPlayerMoveStart()
	PRIVATE_TABLE.moveing = true
end

function Addon:OnPlayerMoveStop()
	PRIVATE_TABLE.moveing = false
end

function Addon:OnMouseDown(frame, button, down)
	if button == "RightButton" then
		if PRIVATE_TABLE.lastClicked then
			local doubleTime = GetTime() - PRIVATE_TABLE.lastClicked
			if doubleTime < ACTION_DOUBLE_WAIT and doubleTime > MIN_ACTION_DOUBLECLICK then
				PRIVATE_TABLE.lastClicked = nil
				if not PRIVATE_TABLE.moveing and not IsMounted() and not PRIVATE_TABLE.inCombat and not InCombatLockdown() then
					if IsMouselooking() then
						MouselookStop()
					end
					SetOverrideBindingClick(PRIVATE_TABLE.secureActionButton, true, "BUTTON2", "LootARangMatic")
					PRIVATE_TABLE.bindingOn = true
					PRIVATE_TABLE.setBindingOn = true
				end
			elseif doubleTime >= ACTION_DOUBLE_WAIT then
				PRIVATE_TABLE.lastClicked = GetTime()
			end
		else
			PRIVATE_TABLE.lastClicked = GetTime()
		end
	 end
end